/**
 * @file defs.h
 * 
 * @author Michal Kopecky  POLiTE library
 * @author Jan Hadrava
 * @author Petr Cermak
 */


#ifndef IOPCDEFS_H
#define IOPCDEFS_H

#include "defsPlatform.h"


////////////////////////////////////////////////////////////////////////////
// Temporary definitions - for development only
#define NOTDEF


/////////////////////////////////////////////////////////////////////////////
// POLiTe default settings

//#define NO_DATABASE_CASCADE_DELETE
//if defined, object is deleted piece by piece from each table
//if not defined, object is deleted only from the main table

#define AUTO_VIRTUAL_OBJECT_LOAD
//if defined, query results provide an automatic object virtualisation

#define SQL_EXECUTION_VERSION_6 0x00
#define SQL_EXECUTION_VERSION_7 0x02
#define SQL_EXECUTION_NATIVE    0x01
//defines various compatibility modes of SQL statement execution

#define SQL_EXECUTION_MODE SQL_EXECUTION_NATIVE
//defines the SQL statement execution mode used in the GEN.LIB

/////////////////////////////////////////////////////////////////////////////
// Name of table associated with PersistentObject class

#define PERSISTENT_OBJECT_TABLE "OID_ROOT"

/////////////////////////////////////////////////////////////////////////////
// Maximal length of the PERSISTENT_OBJECT_TABLE.CLASS_NAME column
// Used for the RefBase virtualisation

#define MAX_CLASS_NAME_LEN 64

/////////////////////////////////////////////////////////////////////////////
// Defines number of lines fetched, when no database is available
// Used in ResultBase::Next(), ifdef NO_DATABASE_PRESENT

#define MAX_FETCHES_FROM_DUMMY_DATABASE 3

/////////////////////////////////////////////////////////////////////////////
// Defines the alias for the name of table associated with object
// Instead of TPERSON.COLUMN it is possible to use #THIS.COLUMN

#define THIS_TABLE "#THIS"

/////////////////////////////////////////////////////////////////////////////
// Defines the prefixes of aliases used for tables and relations
// in Chained relations

#define R_PREFIX "R$_"
#define T_PREFIX "T$_"
#define X_PREFIX "X$_"

/////////////////////////////////////////////////////////////////////////////
// POLiTe default settings

#define DEFAULT_UPDATING_STRATEGY US_OnDemand
#define DEFAULT_LOCKING_STRATEGY LS_None
#define DEFAULT_WAITING_STRATEGY WS_Nowait
#define DEFAULT_READING_STRATEGY RS_Cache
#define DEFAULT_STRATEGIES (DEFAULT_UPDATING_STRATEGY|DEFAULT_LOCKING_STRATEGY|DEFAULT_WAITING_STRATEGY|DEFAULT_READING_STRATEGY)



#ifdef WIN32
#define WINCODE(code)	code
#define LINCODE(code)
#else
#define WINCODE(code)
#define LINCODE(code)	code
#endif /*WIN32*/

#define MAXLONG		((long)(-1)&~(MINLONG))
#define MINLONG		((long)(1 << (8*sizeof(long) - 1)))

//#define NULL 0

#endif //__L_DEFS_H__
