/**
 * @file rwLock.cpp
 * 
 * @author Jan Hadrava
 * @author Petr Cermak
 */

#include <pthread.h>
#include <errno.h>

#include "rwLock.h"
#include "trace.h"
#include "exceptionBase.h"

namespace iopc {

RWLock::RWLock()
	throw (IopcNoMemoryException, IopcRsrcLackException)
{
	switch (pthread_rwlock_init(&rwlock, NULL)) {
	case 0:
		return;
	break;
	case ENOMEM:
		IOPC_LOG_THROW(IopcNoMemoryException());
	break;
	case EAGAIN:
	case EPERM:
		IOPC_LOG_THROW(IopcRsrcLackException());
	break;
	default:
		IOPC_UNREACHABLE;
	break;
	}
}

RWLock::~RWLock()
	throw ()
{
	int res = pthread_rwlock_destroy(&rwlock); IOPC_ASSERT(res == 0);
}

void RWLock::rlock()
	throw (IopcRsrcLackException, IopcDeadlockException)
{
	switch (pthread_rwlock_rdlock(&rwlock)) {
	case 0:
		return;
	break;
	case EAGAIN:
		IOPC_LOG_THROW(IopcRsrcLackException());
	break;
	case EDEADLK:
		IOPC_LOG_THROW(IopcDeadlockException());
	break;
	default:
		IOPC_UNREACHABLE;
	break;
	}
}

void RWLock::wlock()
	throw (IopcDeadlockException)
{
	switch (pthread_rwlock_wrlock(&rwlock)) {
	case 0:
		return;
	break;
	case EDEADLK:
		IOPC_LOG_THROW(IopcDeadlockException());
	break;
	default:
		IOPC_UNREACHABLE;
	break;
	}
}

void RWLock::unlock()
	throw (IopcIncorrectUsageException)
{
	switch (pthread_rwlock_unlock(&rwlock)) {
	case 0:
		return;
	break;
	case EPERM:
		IOPC_LOG_THROW(IopcIncorrectUsageException());
	break;
	default:
		IOPC_UNREACHABLE;
	break;
	}
}

}/*namespace polite*/
