/**
 * @file rwLock.h
 * 
 * @author Jan Hadrava
 * @author Petr Cermak
 */

#ifndef RWLOCK_H
#define RWLOCK_H

#include <pthread.h>
#include "exceptionBase.h"

namespace iopc
{

/** Encapsulated read/write lock */
class DLL_EXTERNAL RWLock {
public:
	RWLock()
		throw (IopcNoMemoryException, IopcRsrcLackException);
	~RWLock()
		throw ();
	/** Waits for lock for reading. */
	void rlock()
		throw (IopcRsrcLackException, IopcDeadlockException);
	/** Waits for lock for writing. */
	void wlock()
		throw (IopcDeadlockException);
	/** Removes one lock. */
	void unlock()
		throw (IopcIncorrectUsageException);
private:	/* hiding */
	RWLock(const RWLock &);
	//RWLock& operator=(const RWLock &);
protected:
	pthread_rwlock_t rwlock;	/**< lock proper. */
};

}

#endif /*RWLOCK_H*/
