#ifndef RWXLOCK_H_
#define RWXLOCK_H_

#include <pthread.h>

#include "exceptionBase.h"
#include "mutex.h"
#include "defs.h"

namespace iopc {

/** Recursive read/write lock. Allows several write locks to be held by a thread. */
class DLL_EXTERNAL RWXLock {
public:
	RWXLock()
		throw (IopcNoMemoryException, IopcRsrcLackException);
	~RWXLock()
		throw ();
	/** Waits lock for reading. */
	void rlock()
		throw (IopcRsrcLackException, IopcDeadlockException, IopcNoMemoryException);
	/** Waits for lock writing. */
	void wlock()
		throw (IopcDeadlockException, IopcNoMemoryException);
	/** Waits for exclusive lock (only one is allowed). */
	void xlock()
		throw (IopcDeadlockException, IopcNoMemoryException);
	/** Tries to get lock for reading immediately. */
	bool tryRLock()
		throw (IopcRsrcLackException, IopcDeadlockException);
	/** Tries to get lock for writing immediately. */
	bool tryWLock()
		throw (IopcRsrcLackException, IopcDeadlockException);
	/** Tries to get exclusive lock immediately. */
	bool tryXLock()
		throw (IopcDeadlockException);
	/** Removes one lock. */
	void unlock()
		throw (IopcIncorrectUsageException);
private:
	RWXLock(const RWXLock& );
	RWXLock& operator=(const RWXLock& );
	
	int wlocks;	/**< Number of locks for writing. */
	bool isXLock;	/**< Indicates whether exclusive lock is locked. */
	pthread_rwlock_t rwlock;
	Mutex mutex;
	pthread_t wlocker;	/**< Owner of locks for writing. */
};

}

#endif /*RWXLOCK_H_*/
