#include <malloc.h>

#include "defs.h"
#include "timeStamp.h"

namespace iopc {

#ifdef WIN32

int gettimeofday(struct timeval *tv, void *)
{
	struct _timeb t;
	_ftime(&t);
	tv->tv_sec = t.time;
	tv->tv_usec = 1000*t.millitm;
	return 0;
}

#endif /*WIN32*/

TimeStamp::TimeStamp()
{
	tv.tv_sec = 0;
	tv.tv_usec = 0;
}

TimeStamp::~TimeStamp()
{
}

void TimeStamp::normalize()
{
	tv.tv_sec += tv.tv_usec/1000000;
	tv.tv_usec %= 1000000;
}

bool TimeStamp::operator==(const TimeStamp &ts) const
{
	return tv.tv_sec == ts.tv.tv_sec && tv.tv_usec == ts.tv.tv_usec;
}

bool TimeStamp::operator!=(const TimeStamp &ts) const
{
	return !(*this == ts);
}

bool TimeStamp::operator<(const TimeStamp &ts) const
{
	return tv.tv_sec < ts.tv.tv_sec || (tv.tv_sec == ts.tv.tv_sec && tv.tv_usec < ts.tv.tv_usec);
}

bool TimeStamp::operator>(const TimeStamp &ts) const
{
	return !(ts < *this);
}

bool TimeStamp::operator<=(const TimeStamp &ts) const
{
	return !(ts < *this);
}

bool TimeStamp::operator>=(const TimeStamp &ts) const
{
	return !(*this < ts);
}

/* compute difference of two time stamps in millseconds */
long TimeStamp::operator-(const TimeStamp &ts) const
{
	long sdiff = tv.tv_sec - ts.tv.tv_sec;
	long udiff = tv.tv_usec - ts.tv.tv_usec;
	if (sdiff <= MINLONG/1000 + 1) {
		return MINLONG;
	} else  if (sdiff >= MAXLONG/1000 - 1) {
		return MAXLONG;
	}
	return 1000*sdiff + udiff/1000;	/* return difference in milliseconds */
}

TimeStamp TimeStamp::operator+(long ms) const
{
	TimeStamp retval = *this;
	retval.tv.tv_sec += ms/1000;
	retval.tv.tv_usec += 1000*(ms%1000);
	retval.normalize();
	return retval;
}

TimeStamp::operator TimeSpec() const
{
	struct timespec tspec;
	tspec.tv_sec = tv.tv_sec;
	tspec.tv_nsec = 1000*tv.tv_usec;
	return tspec;
}

TimeStamp TimeStamp::now()
{
	TimeStamp ts;
	while (gettimeofday(&ts.tv, NULL) != 0);
	ts.normalize();
	return ts;
}

}

