/**
* @file trace.cpp
*
* @author Jiri Kopecky  POLiTE library
* @author Jan Hadrava
* @author Petr Cermak
*/

#include <fstream>
#include <ostream>
#include <sstream>
#include <string>
#include <iostream>
//#include "defs.h"
#include "trace.h"
#include "util.h"

#include "mutex.h"

namespace iopc
{

using namespace std;

// ----- Default masks
#ifdef IOPC_DEBUG
#ifdef IOPC_TRACE
const int LogWriter::defaultTraceMask = ( LogWriter::INFO_SQL | LogWriter::INFO | LogWriter::WARNING | LogWriter::ERROR | LogWriter::DEBUG | LogWriter::TRACE );
const int LogWriter::defaultTraceMaskTTY = ( LogWriter::INFO | LogWriter::WARNING | LogWriter::ERROR | LogWriter::DEBUG );
#else
const int LogWriter::defaultTraceMask = ( LogWriter::INFO | LogWriter::WARNING | LogWriter::ERROR | LogWriter::DEBUG );
const int LogWriter::defaultTraceMaskTTY = ( LogWriter::INFO | LogWriter::WARNING | LogWriter::ERROR | LogWriter::DEBUG );
#endif
#else // #ifdef IOPC_DEBUG
#ifdef IOPC_TRACE
const int LogWriter::defaultTraceMask = ( LogWriter::INFO_SQL | LogWriter::INFO | LogWriter::WARNING | LogWriter::ERROR | LogWriter::TRACE );
const int LogWriter::defaultTraceMaskTTY = ( LogWriter::INFO | LogWriter::WARNING | LogWriter::ERROR );
#else
const int LogWriter::defaultTraceMask = ( LogWriter::INFO | LogWriter::WARNING | LogWriter::ERROR );
const int LogWriter::defaultTraceMaskTTY = ( LogWriter::INFO | LogWriter::WARNING | LogWriter::ERROR );
#endif
#endif // #ifdef IOPC_DEBUG


const string LogWriter::defaultFileName = "iopc_log.txt";

LogWriter* LogWriter::instance = NULL;
Mutex LogWriter::logLock;


void LogWriter::initInstance()
{
	instance = new LogWriter();
}

void LogWriter::terminateInstance()
{
	delete instance;
}

LogWriter& LogWriter::getInstance()
{
	if (instance == NULL) {
		throw new IopcInvalidStateException("Iopc was not initialized yet. Call Iopc::init()");
	}
	return *instance;
}

LogWriter::LogWriter() : traceMask(defaultTraceMask), traceMaskTTY(defaultTraceMaskTTY)
// opens standard log file
{

};

void LogWriter::logOpenInst(const string& fname, bool append)
{
	logClose();

	fileName = fname;

	logStream.open(fileName.c_str(), ofstream::out | (append ? ofstream::app : ofstream::trunc));
	if (!logStream) {
		cerr << "Couldn't open log file: " << fileName << ". Logging to file disabled." << endl;
		return;
	}
	logInfo(" --- Log file opened ---");
}

void LogWriter::logCloseInst()
{
	if (logStream.is_open()) {
		logInfo(" --- Log file closed ---");
		logStream.close();
	}
}

LogWriter::~LogWriter()
{
	logClose();
};

/*int LogWriter::va_log( MsgType mask, const char *format, va_list argp )
{
	int retval = 0;
	if ( ( int ) mask & traceMaskTTY ) {
		logLock.lock();
		retval = vfprintf( stdout, format, argp ) +
		         fprintf( stdout, "\n" );
		logLock.unlock();
	}
	if ( ( ( int ) mask & traceMask ) && ( stdlog != NULL ) ) {
		logLock.lock();
		retval = vfprintf( stdlog, format, argp )
		         + fprintf( stdlog, "\n" );
		logLock.unlock();
	} else {
		retval = 0;
	}
	return retval;
}*/

void LogWriter::logMsgInst(MsgType mask, const string& message)
{
	if ((int)mask & traceMaskTTY) {
		logLock.lock();
		cout << getMsgTypeStr(mask) << ": " << message << endl;
		logLock.unlock();
	}
	if (((int)mask & traceMask) && logStream.good()) {
		logLock.lock();
		logStream << Util::currentTime() << " " << getMsgTypeStr(mask) << ": " << message << endl;
		logLock.unlock();
	}
}

void LogWriter::logLineInst(MsgType mask, string file, uint line, string function, const string& message)
{
	ostringstream lineStream;
	lineStream << file << ", " << line << ", " << function << ", " << message;
	logMsg(mask, lineStream.str());
}

/*int LogWriter::logmsg(const char *format, ...)
{
	va_list argp;

	va_start(argp, format);
	int retval = va_log(TL_INFO, format, argp);
	va_end(argp);
	return retval;
}*/

void LogWriter::logFlushInst()
{
	cerr.flush();
	if (logStream.good()){
		logStream.flush();
	}
}

string LogWriter::getMsgTypeStr(MsgType msgType)
{
	switch (msgType) {
		case INFO_SQL: return "SQL";
		case INFO: return "INF";
		case WARNING: return "WAR";
		case ERROR: return "ERR";
		case TRACE: return "TRC";
		case DEBUG: return "DBG";
		default:
			return "MSG";
	}
}


} /*namespace iopc*/

//#endif //IOPC_TRACE
