#ifndef CONNECTION_H_
#define CONNECTION_H_

#include <string>
#include "iopccommon/defs.h"
/*#include "db/cursor.h"
#include "db/connectionImpl.h"
#include "db/database.h"
*/

namespace iopc {

class Cursor;
class Database;
class ConnectionImpl;
class Driver;

/**
 * The connection interface
 */
class DLL_EXTERNAL Connection
{
public:
	virtual ~Connection() {};

	/** Passes an SQL command to DB. */
	virtual Cursor* sql(const std::string& name) = 0;
	
	//virtual Cursor* getCursor(const std::string& name) = 0;
	
	virtual void returnCursor(Cursor* cur) = 0;
	
	virtual void closeAllCursors() = 0;
	
	virtual void sqlNonQuery(const std::string& name) = 0;
	
	/** Commits current transaction. Starts new one. */
	virtual void commit() = 0;
	/** Rollbacks current transaction.*/
	virtual void rollback() = 0;
	/** Creates a savepoint. */
	virtual void savepoint(const std::string& name) = 0;
	/** Rollbacks to savepoint. */
	virtual void rollbackToSavepoint(const std::string& name) = 0;
	/** Opens connection */
	virtual void open() = 0;
	/** Closes connection */
	virtual void close() = 0;
	/** Commits last transaction and disconnects from DB. */
	virtual void disconnect() = 0;
	/** Rollbacks current trans
	 * action and disconnects from DB. */
	virtual void abort() = 0;
	/** Turns on/off autocommit. */
	virtual void setAutoCommit(bool ac) = 0;
	/** Returns state of this connection */
	virtual bool isOpened() = 0;
	/** Passes an SQL command to DB. */
	virtual Connection& operator<< (
	    const std::string& command
	) = 0;
	
	virtual bool operator== (const Connection& dbConn) const = 0;

	virtual bool operator!= (const Connection& dbConn) const = 0;
	
	virtual Database& getDatabase() = 0;
	
	virtual Driver& getDriver() = 0;
	
	virtual bool isAutoCommit() = 0;
	
	virtual ConnectionImpl* getImplementation() = 0;
	
	virtual Connection* getBasicConnection() = 0;
};

}

#endif /*CONNECTION_H_*/
