/**
 * @file oracle10gCursorImpl.h
 * 
 * @author Josef Troch Josef Troch (<a href="mailto:josef.troch@email.cz">josef.troch@email.cz</a>) 2003-2004
 * @author Petr Cermak
 */

#ifndef ORACLE10GCURSOR_H_
#define ORACLE10GCURSOR_H_

#include "oracle10gOci.h"
#include "iopccommon/defsPlatform.h"
#include "iopcmeta/type.h"
#include "iopcdb/cursorImpl.h"
#include "iopccommon/metadataHolder.h"
#include <string>
#include <map>
#include <utility>

namespace iopc
{

class DLL_EXTERNAL Oracle10gCursorImpl : public CursorImpl
{

public:
	//Oracle10gCursorImpl(const std::string& sqlCommand);
	Oracle10gCursorImpl(OCIEnv* ociEnv, OCIError* ociErr, OCISvcCtx* ociSvc, bool isAutoCommit, const std::string& sqlCommand);
	virtual ~Oracle10gCursorImpl();

	/*! "Opens the cursor." Oracle10g doesn't use cursors, statement handle (\ref stmtHandle) is used instead. 
			So this function only initializes some variables.
	 */
	virtual void open();
	
	/*! Prepares \p sqlCommand to be executed by Oracle (as a result \ref stmtHandle is initialized).
			\param sqlCommand SQL statement to be processed.
	 */
	virtual void prepare();
	
	virtual void preExecBind(const int pos, const void* varAddr, unsigned int size, const Type& dataType, bool setNull, const MetadataHolder& attributes = MetadataHolder::emptyMeta);
	
	virtual void execute();
	
	virtual void preFetchBind(const int pos, const void* varAddr, unsigned int size, const Type& varType, bool* isNull = NULL, const MetadataHolder& attributes = MetadataHolder::emptyMeta);

	virtual bool fetchNext();
	
	virtual bool fetchPrev();
	
	virtual long getPosition();
	
	virtual void close();

	//! Clears \ref indicatorVariables - called from _Close() and from destructor.
	void clear();

//----------------------------------------------------------------------------------
// Types:
	/*! \brief Pair of indicator variable and pointer to bool that should be set according to the 
						 indicator variable after execution of appropriate SQL statement.
	 */
	struct IndicatorStruct	
	{
		sb2 * indicator;			//!< The indicator variable.
		bool * boolPtr;			 //!< The pointer to bool.
		OCIDefine* ociDef;		//!< The OCI Define handle of the define.
	};

//Attributes:
	
	//! OCI statement handle - used for SQL statement processing.
	OCIStmt * stmtHandle;				


	/*! Are we processing a SELECT or some other SQL statement (passed from _Prepare() to _Execute() for setting 
			parameter 'iters' in OCIStmtExecute())?
	 */
	bool isSelectStatement;			

	/*! Pointer to OCI Environment handle (see OciEnvCreate() in Oracle documentation).

			\note Created in Oracle10gDatabase::Orecle10gDatabase() and copied here in constructor of this class.
	 */
	OCIEnv* ociEnvironment; 
	 
	/*! Pointer to OCI Error handle to be passed to OCIErrorGet() for diagnostic information in the event of an error.

			\note Created in Oracle10gDatabase::Orecle10gDatabase() and copied here in constructor of this class.
	 */
	OCIError* errorHandle;		 
	
	/*! The service context pointer.

			\note Created in Oracle10gDatabase::Orecle10gDatabase() and copied here in constructor of this class.
	 */
	OCISvcCtx* serviceContext;

	bool autoCommit;

	unsigned int position;			 //!< Position of cursor in query results (how many rows have been already fetched).
	
	std::string sqlCommand;


	/*! Used to set bool given as parameter \p null in 
			_PreFetchBind(const int,const void *,const int,const IopcDataType,bool *) to appropriate 
			value after fetching values from database.. The list is filled in 
			_PreFetchBind(const int,const void *,const int,const IopcDataType,bool *) and processed in 
			_FetchNext().
	 */
	std::map<int, IndicatorStruct*> indicatorVariables;
	
	void test();
	
	
private:
	inline const std::string getOCIMsg(
		int ociRetVal,
		const std::string funcName,
		bool useEnvHandle = false) {
			
		return 	iopc::getOCIMsg(ociEnvironment, errorHandle, ociRetVal, funcName, useEnvHandle);
	}
	
	inline const std::pair<const std::string, const ub4> getOCIError(
		int ociRetVal,
		const std::string funcName,
		bool useEnvHandle = false) {
			
		return iopc::getOCIError(ociEnvironment, errorHandle, ociRetVal, funcName, useEnvHandle);
	}
	
	//static ub2 getOracleType(const Type& varType);

	
};


} // namespace

#endif /*ORACLE10GCURSOR_H_*/
