#include "oracle10gSqlStatementsFeature.h"
#include "iopccommon/trace.h"
#include <sstream>

using namespace iopc;
using namespace std;
/*
const MetadataHolder& Oracle10gSqlStatementsFeature::getRootType(const MetadataHolder& type)
{
	return *((const MetadataHolder*) type.getMeta("db.oracle.oracleObjects.root").getPointerValue());
}
*/
string Oracle10gSqlStatementsFeature::getCreateTableStatement(
	const MetadataHolder& tableMeta, 
	const vector<const MetadataHolder*>& columns, 
	const MetadataHolder& createMeta)
{
	IOPC_FUNC_INVOKED;
	ostringstream ostr;
	
	if (tableMeta.getMeta("db.mapping.type").getStringValue() == "object") {
		ostr << "CREATE TABLE " << tableMeta.getMeta("db.table").getStringValue();
		ostr << " OF " << tableMeta.getMeta("db.type").getStringValue();
		ostr << " ( " << getColumns(tableMeta, columns, false, true, true) << " )"; 
		return ostr.str();
	}
	
	ostr << "CREATE TABLE " << tableMeta.getMeta("db.table").getStringValue() << " ( ";
	ostr << getColumns(tableMeta, columns, true, true, true);
	ostr << ")";
	
	return ostr.str();
}

vector<string> Oracle10gSqlStatementsFeature::getIndexes(
	const MetadataHolder& tableMeta,
	const vector<const MetadataHolder*>& columns)
{
	vector<string> ret;
	for (vector<const MetadataHolder*>::const_iterator it = columns.begin(); it != columns.end(); it++) {
		if (!(*it)->hasMeta("db.index") && !(*it)->hasMeta("db.index.unique")) continue;
		
		string indexName;
		bool unique = false;
		ostringstream ostr;
		if ((*it)->hasMeta("db.index")) {
			indexName = (*it)->getMeta("db.index").getStringValue();
		} else {
			indexName = (*it)->getMeta("db.index.unique").getStringValue();
			unique = true;
		}
		ostr << "CREATE ";
		if (unique) ostr << "UNIQUE ";
		ostr << "INDEX " << tableMeta.getMeta("db.table").getStringValue() << "_" << indexName << " ON " << tableMeta.getMeta("db.table").getStringValue();
		
		if (! tableMeta.hasMeta("db.type")) { 
			ostr << " (" << (*it)->getMeta("db.column").getStringValue() << ")";
		} else {
			ostr << " X (TREAT (VALUE(X) AS " << tableMeta.getMeta("db.type").getStringValue() << ").";
			ostr <<  (*it)->getMeta("db.column").getStringValue() << ")";
		}
		
		if (tableMeta.hasFeature("db.index.computeStatistics." + indexName)) {
			ostr << " COMPUTE STATISTICS";
		}
		ret.push_back(ostr.str());
	}
	return ret;
}

string Oracle10gSqlStatementsFeature::getColumns(
	const MetadataHolder& tableMeta, 
	const vector<const MetadataHolder*>& columns, bool doColumns, bool doConstraints, bool doTypes, bool typesAsParams, bool useViewColumns, string prefix)
{
	ostringstream ostr;
	string primaryKeyCols;
	bool first = true;
	for (vector<const MetadataHolder*>::const_iterator it = columns.begin(); it != columns.end(); it++){
		
		const MetadataHolder& meta = *(*it);
		
		if (doColumns) {
			if (it != columns.begin()) {
				ostr << ", ";
			}
			first = false;
			if (useViewColumns) {
				ostr << prefix << meta.getMeta("db.viewColumn").getStringValue() ;
			} else {
				ostr << prefix << meta.getMeta("db.column").getStringValue() ;
			}
			if (doTypes) {
				if (typesAsParams) {
					ostr << " " << meta.getMeta("db.oracle.type.sql.param").getStringValue();
				} else {
					ostr << " " << meta.getMeta("db.oracle.type.sql").getStringValue();
				}
				if (meta.hasFeature("db.type.notNull")) {
					ostr << " NOT NULL";
				}
			}
		}
		if (doConstraints) {
			if (meta.hasFeature("db.primaryKey")) {
				if (primaryKeyCols != "") {
					primaryKeyCols += ", ";
				}
				primaryKeyCols += meta.getMeta("db.column").getStringValue();
			}
		}
	}
	if (doConstraints && primaryKeyCols != "") {
		if (!first) {
			ostr << ", ";
		}
		ostr << "CONSTRAINT " << tableMeta.getMeta("db.table").getStringValue() << "_pk " << " PRIMARY KEY (" << primaryKeyCols << ")";
	}
	return ostr.str();
}

string Oracle10gSqlStatementsFeature::getColumnsBindStr(
	const vector<const MetadataHolder*>& inColumns,
	bool doColNames,
	string prefix)
{
	ostringstream ostr;
	int i = 1;
	for (vector<const MetadataHolder*>::const_iterator it = inColumns.begin(); it != inColumns.end(); it++, i++){
		if (i != 1) {
			ostr << ", ";
		}
		if (doColNames) {
			ostr << prefix << (*it)->getMeta("db.column").getStringValue() << "=>";
		}
		ostr << ":v" << i; 
	}
	return ostr.str();
}

string Oracle10gSqlStatementsFeature::getDropTableStatement(
	const MetadataHolder& tableMeta, 
	const MetadataHolder& dropMeta)
{
	ostringstream ostr;
	ostr << "DROP TABLE " << tableMeta.getMeta("db.table").getStringValue();
	if (dropMeta.hasFeature("db.oracle.drop.purge")) {
		ostr << " PURGE";
	}
	return ostr.str();
}

string Oracle10gSqlStatementsFeature::getDropViewStatement(
	const MetadataHolder& viewMeta, 
	const MetadataHolder& dropMeta)
{
	ostringstream ostr;
	ostr << "DROP VIEW " << viewMeta.getMeta("db.view").getStringValue();
	if (dropMeta.hasFeature("db.oracle.drop.purge")) {
		ostr << " PURGE";
	}
	return ostr.str();
}

string Oracle10gSqlStatementsFeature::getInsertStatement(
	const MetadataHolder& tableMeta,
	const vector<const MetadataHolder*>& inColumns,
	const MetadataHolder& insertMeta)
{
	ostringstream ostr;
	ostr << "INSERT INTO ";
	if (tableMeta.hasFeature("db.oracle.useOracleObjects"))
	{
		// TODO: OO INSERT
		//ostr << getRootType(tableMeta).getMeta("db.table").getStringValue();
		ostr << " VALUES " << tableMeta.getMeta("db.oracle.oracleObjects.type").getStringValue() << "(";
		ostr << getColumnsBindStr(inColumns) << ")";
	} else {
		ostr << tableMeta.getMeta("db.table").getStringValue() << " (";
		ostr << getColumns(tableMeta, inColumns, true, false, false);
		ostr << ") VALUES (" << getColumnsBindStr(inColumns) << ")";
	} 
	IOPC_DEBUGLOG(ostr.str());
	return ostr.str();
}

string Oracle10gSqlStatementsFeature::getUpdateStatement(
	const MetadataHolder& tableMeta,
	const vector<const MetadataHolder*>& inColumns,
	const MetadataHolder& updateMeta)
{
	ostringstream ostr;
	ostringstream values;
	ostr << "UPDATE " << tableMeta.getMeta("db.table").getStringValue() << " SET ";	
	int i = 1;
	for (vector<const MetadataHolder*>::const_iterator it = inColumns.begin(); it != inColumns.end(); it++, i++) {
		if (it != inColumns.begin()) {
			ostr << ", ";
		}
		const MetadataHolder& meta = *(*it);
		ostr << meta.getMeta("db.column").getStringValue() << " = :v" << i;
	}
	if (updateMeta.hasMeta("db.update.where")) {
		ostr << " WHERE " << updateMeta.getMeta("db.update.where").getStringValue();
	}
	
	return ostr.str();
}

string Oracle10gSqlStatementsFeature::getDeleteStatement(
	const MetadataHolder& tableMeta,
	const MetadataHolder& deleteMeta)
{
	ostringstream ostr;
	ostr << "DELETE FROM " << tableMeta.getMeta("db.table").getStringValue() << " WHERE " << deleteMeta.getMeta("db.delete.where").getStringValue();
	return ostr.str();
}

string Oracle10gSqlStatementsFeature::getSelectStatement(
	const vector<const MetadataHolder*>& tables,
	const vector<const MetadataHolder*>& outColumns,
	const MetadataHolder& selectMeta)
{
	ostringstream ostr;
	ostr << "SELECT ";
	for (vector<const MetadataHolder*>::const_iterator it = outColumns.begin(); it != outColumns.end(); it++) {
		if (it != outColumns.begin()) {
			ostr << ", ";
		}
		const MetadataHolder& meta = *(*it);
		ostr << meta.getMeta("db.column").getStringValue();
		if (meta.hasMeta("db.column.alias")) {
			ostr << " AS " << meta.getMeta("db.column.alias").getStringValue();
		}
	}
	ostr << " FROM ";
	for (vector<const MetadataHolder*>::const_iterator it = tables.begin(); it != tables.end(); it++) {
		if (it != tables.begin()) {
			ostr << ", ";
		}
		const MetadataHolder& meta = *(*it);
		ostr << meta.getMeta("db.table").getStringValue();
	}
		
	if (selectMeta.hasMeta("db.select.where")) {
		ostr << " WHERE " << selectMeta.getMeta("db.select.where").getStringValue();
	}
	
	if (selectMeta.hasMeta("db.select.groupBy")) {
		ostr << " GROUP BY " << selectMeta.getMeta("db.select.groupBy").getStringValue();
	}
	
	if (selectMeta.hasMeta("db.select.having")) {
		ostr << " HAVING " << selectMeta.getMeta("db.select.having").getStringValue();
	}
	if (selectMeta.hasMeta("db.select.orderBy")) {
		ostr << " ORDER BY " << selectMeta.getMeta("db.select.orderBy").getStringValue();
	}
	
	if (selectMeta.hasFeature("db.select.lock")) {
		ostr << " FOR UPDATE";
		if (selectMeta.hasFeature("db.select.nowait")) {
			ostr << " NOWAIT";
		}
		
	}
	
	return ostr.str();
}

string Oracle10gSqlStatementsFeature::getCreateViewStatement(
	const MetadataHolder& viewMeta,
	const MetadataHolder& createMeta)
{
	ostringstream ostr;
	ostr << "CREATE VIEW " << viewMeta.getMeta("db.view.name").getStringValue() << " AS (" << viewMeta.getMeta("db.view.select").getStringValue() << ")";
	
	return ostr.str();
}
