#include "oracle10gORStatementsFeature.h"
#include <string>
#include "iopcmeta/type.h"
#include "iopccommon/globalSettings.h"
#include <sstream>
#include "iopclib/types/dbType.h"
#include <set>
#include "iopccommon/util.h"

using namespace iopc;
using namespace std;

bool Oracle10gORStatementsFeature::validate(ORMappingStrategy& currentStrategy, string& msg)
{
	return true;
}

string Oracle10gORStatementsFeature::getInsertStatement(ORMappingStrategy& currentStrategy,  const MetadataHolder& attributes, bool useCache)
{
	if (useCache && cache.hasCachedStatement(currentStrategy.getType().getQualifiedName() + cache.insertStatementType)) {
		return cache.getCachedStatement(currentStrategy.getType().getQualifiedName() + cache.insertStatementType);
	}
	string stat;
	vector<const MetadataHolder*> cols;
	for (ORMappingStrategy::MappedAttributesIterator it = currentStrategy.getMappedAttributes().begin(); it != currentStrategy.getMappedAttributes().end(); it++) {
		cols.push_back(it->first);
	}
	stat = sqlStatements->getInsertStatement(currentStrategy.getType(), cols);
	if (useCache) {
		cache.setCachedStatement(currentStrategy.getType().getQualifiedName() + cache.insertStatementType, stat);
	}
	return stat;
}


string Oracle10gORStatementsFeature::getUpdateStatement(ORMappingStrategy& currentStrategy, const ORMappingStrategy::MappedAttributesList& updateCols,  const MetadataHolder& attributes, bool useCache)
{
	MetadataHolder updateMeta;
	vector<const MetadataHolder*> cols;
	for (ORMappingStrategy::MappedAttributesIterator it = updateCols.begin(); it != updateCols.end(); it++) {
		if (it->second != ORMappingStrategy::COLSOURCE_INHERITEDPK) {
			cols.push_back(it->first);
		}
	}

	//string whereStr = TypeDesc<DatabaseObject>::getType().getAttribute("oid").getMeta("db.column").getStringValue() + " = :oid";
	updateMeta["db.update.where"].setStringValue(getWhere(currentStrategy.getDbType().getKeyAttributes(), "db.column"));

	string ret = sqlStatements->getUpdateStatement(currentStrategy.getType(), cols, updateMeta);
	return ret;
}

string Oracle10gORStatementsFeature::getUpdateStatement(ORMappingStrategy& currentStrategy, const MetadataHolder& attributes, bool useCache)
{
	if (useCache && cache.hasCachedStatement(currentStrategy.getType().getQualifiedName() + cache.updateStatementType)) {
		return cache.getCachedStatement(currentStrategy.getType().getQualifiedName() + cache.updateStatementType);
	}

	string stat = getUpdateStatement(currentStrategy, currentStrategy.getMappedAttributes(), attributes);
	if (useCache) {
		cache.setCachedStatement(currentStrategy.getType().getQualifiedName() + cache.updateStatementType, stat);
	}
	return stat;
}

string Oracle10gORStatementsFeature::getDeleteStatement(ORMappingStrategy& currentStrategy,  const MetadataHolder& attributes, bool useCache)
{
	if (useCache && cache.hasCachedStatement(currentStrategy.getType().getQualifiedName() + cache.deleteStatementType)) {
		return cache.getCachedStatement(currentStrategy.getType().getQualifiedName() + cache.deleteStatementType);
	}
	MetadataHolder deleteMeta;
	deleteMeta["db.delete.where"].setStringValue(getWhere(currentStrategy.getDbType().getKeyAttributes(), "db.column"));

	string stat = sqlStatements->getDeleteStatement(currentStrategy.getType(), deleteMeta);
	if (useCache) {
		cache.setCachedStatement(currentStrategy.getType().getQualifiedName() + cache.deleteStatementType, stat);
	}
	return stat;
}


string Oracle10gORStatementsFeature::getSelectFromClassViewStatement(ORMappingStrategy& currentStrategy, const MetadataHolder& attributes, bool useCache)
{
	if (useCache && cache.hasCachedStatement(currentStrategy.getType().getQualifiedName() + cache.selectViewStatementType)) {
		return cache.getCachedStatement(currentStrategy.getType().getQualifiedName() + cache.selectViewStatementType);
	}

	MetadataHolder tableMeta;
	MetadataHolder selectMeta;
	vector<const MetadataHolder*> cols;
	vector<const MetadataHolder*> tables;
	string whereStr;
	string ret;

	tableMeta["db.table"].setStringValue(currentStrategy.getType().getMeta("db.view.sv").getStringValue());
	tables.push_back(&tableMeta);

	for (DbType::PersistentAttributesIterator it = currentStrategy.getDbType().getAllPersistentAttributes().begin(); it != currentStrategy.getDbType().getAllPersistentAttributes().end(); it++) {
		MetadataHolder* colMeta = new MetadataHolder();
		(*colMeta)["db.column"].setStringValue((*it)->getMeta("db.viewColumn").getStringValue());
		cols.push_back(colMeta);
	}

	//whereStr = TypeDesc<DatabaseObject>::getType().getAttribute("oid").getMeta("db.viewColumn").getStringValue() + " = :oid";
	selectMeta["db.select.where"].setStringValue(getWhere(currentStrategy.getDbType().getKeyAttributes(), "db.viewColumn"));

	ret = sqlStatements->getSelectStatement(tables, cols, selectMeta);

	for (vector<const MetadataHolder*>::const_iterator it = cols.begin(); it != cols.end(); it++) {
		delete *it;
	}

	if (useCache) {
		cache.setCachedStatement(currentStrategy.getType().getQualifiedName() + cache.selectViewStatementType, ret);
	}
	return ret;
}

string Oracle10gORStatementsFeature::getSelectFromClassTableStatement(ORMappingStrategy& currentStrategy, const MetadataHolder& attributes, bool useCache)
{
	string stTypePostfix = "@";
	if (useCache) {
		if (attributes.hasFeature("db.select.lock")) {
			stTypePostfix += "L";
		}
		if (attributes.hasFeature("db.select.nowait")) {
			stTypePostfix += "W";
		}
		if (cache.hasCachedStatement(currentStrategy.getType().getQualifiedName() + cache.selectTableStatementType)) {
			return cache.getCachedStatement(currentStrategy.getType().getQualifiedName() + cache.selectTableStatementType + stTypePostfix);
		}
	}

	MetadataHolder tableMeta;
	MetadataHolder selectMeta;
	vector<const MetadataHolder*> cols;
	vector<const MetadataHolder*> tables;

	tableMeta["db.table"].setStringValue(currentStrategy.getType().getMeta("db.table").getStringValue());
	tables.push_back(&tableMeta);

	for (ORMappingStrategy::MappedAttributesIterator it = currentStrategy.getMappedAttributes().begin(); it != currentStrategy.getMappedAttributes().end(); it++) {
		cols.push_back(it->first);
	}
	selectMeta["db.select.where"].setStringValue(getWhere(currentStrategy.getDbType().getKeyAttributes(), "db.column"));
	selectMeta.setSecondaryMetadataLocation(&attributes);

	string ret = sqlStatements->getSelectStatement(tables, cols, selectMeta);
	if (useCache) {
		cache.setCachedStatement(currentStrategy.getType().getQualifiedName() + cache.selectTableStatementType + stTypePostfix, ret);
	}
	return ret;
}

string Oracle10gORStatementsFeature::getWhere(const DbType::KeyAttributesSet& keys, string metaColName)
{
	ostringstream out;
	int i = 1;
	for (DbType::KeyAttributesIterator it = keys.begin(); it != keys.end(); it++)
	{
		if (it != keys.begin()) {
			out << " AND ";
		}

		out << (*it)->getMeta(metaColName).getStringValue() << " = :key" << i;
		i++;
	}
	return out.str();
}

vector<string> Oracle10gORStatementsFeature::getCreateClassTableScript(ORMappingStrategy& currentStrategy, const MetadataHolder& attributes)
{
	vector<string> ret;
	vector<const MetadataHolder*> colMeta;

	if (!currentStrategy.hasDbTable()) {
		// empty result
		return ret;
	}
	for (ORMappingStrategy::MappedAttributesIterator it = currentStrategy.getMappedAttributes().begin(); it != currentStrategy.getMappedAttributes().end(); it++) {
		string typeSql = typeMapping->getTypeSql(it->first->getType(), *(it->first));
		AdditionalMetadataHolder* amh = new AdditionalMetadataHolder(*it->first);
		/*if (it->second == DbType::COLSOURCE_FILTERED) {
			amh->setFeature("db.type.null");
		}*/
		(*amh)["db.oracle.type.sql"].setStringValue(typeSql);
		colMeta.push_back(amh);
	}

	ret.push_back(sqlStatements->getCreateTableStatement(currentStrategy.getType(), colMeta));

	vector<string> indexes = sqlStatements->getIndexes(currentStrategy.getType(), colMeta);
	if (!indexes.empty()) {
		ret.insert(ret.end(), indexes.begin(), indexes.end());
	}

	for (vector<const MetadataHolder*>::const_iterator it = colMeta.begin(); it != colMeta.end(); it++) {
		delete *it;
	}




	return ret;
}

vector<string> Oracle10gORStatementsFeature::getDropClassTableScript(ORMappingStrategy& currentStrategy,  const MetadataHolder& attributes)
{
	vector<string> ret;
	ret.push_back(sqlStatements->getDropTableStatement(currentStrategy.getType()));
	return ret;
}

void Oracle10gORStatementsFeature::processAttributesForView(ORMappingStrategy& currentStrategy, bool& start, ostringstream& ocolsStr, set<string>& processedColumns, const Type& mainType)
{
	string origTabName;
	if (currentStrategy.getDbType().getMappingType() == DbType::MAPPING_FILTERED) {
		// filtered
		origTabName = currentStrategy.getInsertToType().getType().getMeta("db.table").getStringValue();
	} else {
		// horizontal or vertical
		origTabName = currentStrategy.getType().getMeta("db.table").getStringValue();
	}

	for (ORMappingStrategy::MappedAttributesIterator it = currentStrategy.getMappedAttributes().begin(); it != currentStrategy.getMappedAttributes().end(); it++) {
		const Attribute& attr = *it->first;

		if ((it->second == ORMappingStrategy::COLSOURCE_FILTERED && !mainType.isTypeOf(attr.getOwner())) ||
			it->second == ORMappingStrategy::COLSOURCE_INHERITEDPK ||
			(it->second == ORMappingStrategy::COLSOURCE_INHERITED && currentStrategy.getDbType().getMappingType() != DbType::MAPPING_HORIZONTAL)) {
			// include inherited columns only if the type mapping is horizontal
			// include filtered columns only if they are not from other types than mainType's ancestors (or mainType itself)
			continue;
		}

		// eliminate duplicate columns in the view
		if (processedColumns.find(attr.getMeta("db.viewColumn").getStringValue()) != processedColumns.end()) {
			continue;
		}
		processedColumns.insert(attr.getMeta("db.viewColumn").getStringValue());

		if (!start) {
			ocolsStr << ", ";
		} else {
			start = false;
		}

		ocolsStr << origTabName << "." << attr.getMeta("db.column").getStringValue();
		ocolsStr << " AS " << attr.getMeta("db.viewColumn").getStringValue();
	}
}
