#ifndef LRUCACHEBASE_H_
#define LRUCACHEBASE_H_

// project: POLiTe
// authors: Jan Hadrava 2004

/** \file */

#include "iopccommon/defs.h"
#include "extendedCache.h"
#include "lruCacheItem.h"
#include "types.h"

namespace iopc {

class LruCacheItem;

/** Common predecessor of extended object caches using LRU replacement policy,
* they use LruCacheItem to represent stored objects */
class DLL_EXTERNAL LruCacheBase : public ExtendedCache {
	
	friend class LruCacheItem;

public:
	/** Initialize cache with the same strategies for all object.
	* \param minValue Minimal value of objects obliged to stay in the cache.
	* \param maxCost Maximal total cost of stored objects.
	* \param strat Default strategy.
	*/
	inline LruCacheBase(long minValue = MINLONG, long maxCost = MAXLONG, const Strategy& strat = Strategy::defaultStrategy);
	/** Initialize cache with strategies passed by selector.
	* \param minValue Minimal value of objects obliged to stay in the cache.
	* \param maxCost Maximal total cost of stored objects.
	* \param strat strategy selector.
	**/
	inline LruCacheBase(long minValue, long maxCost, StrategySelector* stratSel);
	inline virtual ~LruCacheBase();
	
private:
	LruCacheBase(LruCacheBase& );
	LruCacheBase &operator=(LruCacheBase& );
	
protected:

	inline LruCacheBase(long minValue, long maxCost, StrategySelector* stratSel, const Strategy& strat);
	
	/** Adds a lock to an item containing an object
	* \param item The item the lock should be added to.
	* \param rdonly Type of the lock.
	* \return An item representing new lock (it's the same as the passed one)
	*/
	virtual LruCacheItem& addLock(LruCacheItem &item, bool rdonly) = 0;
	/** Removes one lock of an item representing an object
	* \param item The item the lock should be removed from.
	*/
	virtual void rmLock(LruCacheItem& item) = 0;
};

inline LruCacheBase::LruCacheBase(long minValue, long maxCost, StrategySelector* stratSel)
: ExtendedCache(minValue, maxCost, stratSel)
{
}

inline LruCacheBase::LruCacheBase(long minValue, long maxCost, const Strategy& strat)
: ExtendedCache(minValue, maxCost, strat)
{
}

inline LruCacheBase::LruCacheBase(long minValue, long maxCost, StrategySelector* stratSel, const Strategy& strat)
: ExtendedCache(minValue, maxCost, stratSel, strat)
{
}

inline LruCacheBase::~LruCacheBase()
{
}

}/*namespace iopc*/


#endif /*LRUCACHEBASE_H_*/
