#ifndef LRUCACHEITEM_H_
#define LRUCACHEITEM_H_

// project: POLiTe
// authors: Jan Hadrava 2004

/** \file */

#include "../databaseObject.h"
#include "../persIdentification.h"
#include "iopccommon/rwxLock.h"
#include "iopccommon/timeStamp.h"
#include "cacheLock.h"
#include "strategy.h"
#include "iopccommon/defs.h"

namespace iopc {

class LruCacheBase;

/** Represents an object stored in cache using LRU strategy. Usually the item is part of a double linked list. */
class DLL_EXTERNAL LruCacheItem : public CacheLock {
public:
	/** Enumeration of item states. */
	enum State {
		ST_BORN,	/**< An object without persistent copy. */
		ST_ADULT,	/**< An object with persistent copy. */
		ST_DOOMED,	/**< Object scheduled for deletion. */
		ST_ZOMBIE,	/**< Remnant of an item. */
		ST_BUSY		/**< Item being processed. */
	};
public:
	LruCacheItem();
	LruCacheItem(LruCacheBase& cache, const Strategy& strat, const PersIdentification persId, DatabaseObject* obj = NULL, State state = LruCacheItem::ST_BORN);
	~LruCacheItem();
	virtual DatabaseObject& getObject();
	virtual CacheLock& addLock(bool rdonly);
	virtual void rmLock();
	/** Replaces the object represented by the item. */
	virtual void replaceObject(DatabaseObject* obj);
private:	/* hiding */
	LruCacheItem(const LruCacheItem& );
	LruCacheItem& operator=(const LruCacheItem& );
public:
	/** Removes item from list. */
	void remove();
	/** Appends an item. */
	void addNext(LruCacheItem& item);
	/** Prepends an item. */
	void addPrev(LruCacheItem& item);
	/** Releases the object being represented by the item. */
	void release();
public:
	RWXLock lock;
	LruCacheBase* cache;	/**< Owner of the item. */
	Strategy strat;
	PersIdentification persId;	/**< ID of the object. */
	DatabaseObject* objectPtr;		/**< The object being represented by the item. */
	State state;			/**< State of the item. */
	LruCacheItem* next;		/**< Next item, */		
	LruCacheItem* prev;		/**< Previous item. */
	bool isLocked;	
	TimeStamp timeStamp;		/**< Time of the last access to the item. */	
};

}/*namespace iopc*/

#endif /*LRUCACHEITEM_H_*/
