#include "statementCache.h"
#include <string>
#include "iopccommon/exceptionBase.h"

using namespace std;

namespace iopc {

string StatementCache::insertStatementType = "@INSERT";
string StatementCache::updateStatementType = "@UPDATE";
string StatementCache::deleteStatementType = "@DELETE";
string StatementCache::selectTableStatementType = "@SELTAB";
string StatementCache::selectViewStatementType = "@SELVIEW";
string StatementCache::selectSerialIdStatementType = "@SELSER";
string StatementCache::selectNextSerialIdStatementType = "@NEXTSER";
string StatementCache::selectTypeStatementType = "@SELTYPE";
string StatementCache::selectNextOidStatementType = "@SELOID";


bool StatementCache::hasCachedStatement(const string& statementType)
{
	statementCacheLock.rlock();
	bool ret = statementCache.find(statementType) != statementCache.end();
	statementCacheLock.unlock(); 
	return ret;
}

string StatementCache::getCachedStatement(const string& statementType)
{
	statementCacheLock.rlock();
	map<string, string>::const_iterator it = statementCache.find(statementType);
	if (it != statementCache.end()) {
		string ret = it->second;
		statementCacheLock.unlock();
		return ret;
	} else {
		statementCacheLock.unlock();
		IOPC_LOG_THROW(IopcNotFoundException("No such statement in the statement cache."));
	}
}

void StatementCache::setCachedStatement(const string& statementType, const string& statement)
{
	statementCacheLock.wlock();
	statementCache[statementType] = statement;
	statementCacheLock.unlock();
}

}
