#ifndef DBTYPE_H_
#define DBTYPE_H_

#include "iopccommon/defsPlatform.h"
#include "../oidObject.h"
#include "iopcdb/connection.h"
#include "iopcmeta/typeDesc.h"
#include "iopccommon/rwLock.h"
#include "../keyValues.h"
#include "mappingStrategy.h"
//#include "meta/complexType.h"
#include <string>
#include <vector>

namespace iopc {

class Driver;
class ObjectMappingStrategy;
class ORMappingStrategy;
	/*
class SqlStatements {
public:
	std::string insertStatement;
	std::string updateStatement;
	std::string selectStatement;
	std::string deleteStatement;
};*/

//class DatabaseObject;

class DLL_EXTERNAL DbType {
	friend class PersIdentification;
public:
	DbType();
	void init(const Type& type);
	void postInit();
//	DbType(const Type& type);
//	DbType(const Type& type);

public:

	typedef std::vector<const Attribute*> PersistentAttributesList;
	typedef PersistentAttributesList::const_iterator PersistentAttributesIterator;

	typedef std::set<const Attribute*> KeyAttributesSet;
	typedef KeyAttributesSet::const_iterator KeyAttributesIterator;

	typedef std::set<DbType*> AllParentsSet;
	typedef std::vector<DbType*> AllParentsList;
	typedef AllParentsList::const_iterator AllParentsIterator;
	typedef AllParentsList::const_reverse_iterator AllParentsReverseIterator;

//	DbType& operator=(const DbType& src);
	inline bool operator!=(const DbType& src) const { return *type != *src.type; }
	inline bool operator==(const DbType& src) const { return *type == *src.type; }
	inline bool operator<(const DbType& src) const { return *type < *src.type; }

	DatabaseObject* loadDatabaseObject(Connection& conn, const KeyValues& keyVals, bool wait, bool lock);

	// Loads DatabaseObject from connectiona
	DatabaseObject* loadDatabaseObject(Cursor& cur);

	OidObject* loadOidObject(Connection& conn, const OidType& oid, bool wait, bool lock);

	void deleteOidObject(Connection& conn, const OidType& oid);

	void deleteDatabaseObject(Connection& conn, const KeyValues& keyVals);

	void updateObject(Connection& conn, DatabaseObject& obj);

	void insertObject(Connection& conn, DatabaseObject& obj);

	KeyValues getKeyValuesForObject(const DatabaseObject& obj) const;

	inline const Type& getType() const { return *type; }

	virtual ~DbType();

	enum Mapping {
		MAPPING_HORIZONTAL =0,
		MAPPING_VERTICAL,
		MAPPING_FILTERED,
		MAPPING_OBJECT
	};

	inline Mapping getMappingType() const { return mapping; }
	inline const AllParentsList& getAllParents() const { return allParents; }
	inline const AllParentsSet& getAllParentsSet() const { return allParentsSet; }
	inline const PersistentAttributesList& getAllPersistentAttributes() const { return allPersistentAttributes; }
	inline const KeyAttributesSet& getKeyAttributes() const { return keyAttributes; }
	inline const PersistentAttributesList& getPersistentAttributes() const { return persistentAttributes; }
	inline const PersistentAttributesList& getInheritedAttributes() const { return inheritedAttributes; }
	inline bool isOidManaged() const { return oidManaged; }
	inline MappingStrategy& getMappingStrategy() const { return *strategy; }
	ObjectMappingStrategy& getObjectMappingStrategy() const;
	ORMappingStrategy& getORMappingStrategy() const;
private:
	//std::map<Driver*, SqlStatements> sqlStatements;
	//static std::map<Driver*, StaticSqlStatements> staticSqlStatements;

	//inline const MappedAttributesList& getMappedAttributes() const { return mappedAttributes; }

	void insertOne(Connection& conn, DatabaseObject& obj);
	void updateOne(Connection& conn, DatabaseObject& obj);
	void deleteOne(Connection& conn, const OidType& oid);
	void deleteOne(Connection& conn, const KeyValues& keyVals);
	void loadOne(Connection& conn, DatabaseObject* obj, const KeyValues& keyVals, bool wait, bool lock);

	//void executeQuery(Connection& conn, std::string query, vector<OidType> result);

	Mapping mapping;

//	DbType& operator=(DbType& src);
private:
	const Type* type;
	bool initialized;

	/** All (even indirect) parents of this type that have representation as a table in the database */
	AllParentsList allParents;
	AllParentsSet allParentsSet;

	/** List of persistent attributes */
	PersistentAttributesList persistentAttributes;

	/** List of inherited attributes */
	PersistentAttributesList inheritedAttributes;

	/** List of all persistent attributes (even inherited) */
	PersistentAttributesList allPersistentAttributes;

	/** List of keys **/
	KeyAttributesSet keyAttributes;


	//RWLock& lock;

	bool oidManaged;

	MappingStrategy* strategy;
};

}


#endif /*DBTYPE_H_*/
