#ifndef METADATAINITIALIZER_H_
#define METADATAINITIALIZER_H_

namespace iopc {

/**
 * @brief Allows users to initialize class metadata at the library startup.
 *
 * Class metadata can be initialized in two standard ways:
 * either by implementing the static void iopcInit(iopc::Type& t)
 * or by passing an instance of a MetadataInitializer implementation
 * to @a IopcMeta::init(MetadataInitializer* metadataInitializer)
 * Only these two ways guarantee that the class metadata will be set properly
 * and that they will have desired effect on the program execution.
 *
 * @author Petr Cermak
 */
class MetadataInitializer {
	public:

		/**
		 * @brief Implementations should set the class metadata here
		 *
		 * Typical implementation would load metadata from a configuration file
		 * and use TypeManager and related classes to set them on matching classes and attributes.
		 */
		virtual void initializeMetadata()=0;

		virtual ~MetadataInitializer() {};
};


}

#endif /* METADATAINITIALIZER_H_ */
