#ifndef OBJECT_H_
#define OBJECT_H_

#include "iopccommon/defsPlatform.h"
#include "iopccommon/trace.h"
#include "type.h"
#include "attributeValue.h"
#include "iopccommon/lock.h"
#include <map>

namespace iopc {

class Type;

class DLL_EXTERNAL Object
{
public:
	typedef std::map<std::string, AttributeValue> AttributeValuesMap;
	typedef AttributeValuesMap::const_iterator AttributeValuesIterator;

private:
	mutable bool metaInitialized;
	bool dirtyInitialized;
	bool dirty;
	mutable AttributeValuesMap attributes;
	mutable const Type* type;
	mutable Lock lock;

public:
	Object();

	void initMeta() const;
	void initDirty();

	virtual std::string toString() const;
	virtual int getMemCost() const;

	/** Checks whether persistent object was modified since it was loaded from persistent data store */
	virtual bool isDirty();
	virtual void resetDirty();
	virtual void setDirty(bool dirty = true);
	/** This function is called whenever persistent attributes are modified. */
/*	inline virtual void setDirty(bool dirty = true) {
		IOPC_DEBUGLOG("Object::setDirty");
		this->dirty = dirty;
	}*/

	virtual int compare(const Object& o) const;

	inline bool operator<(const Object& o) const {
		return (compare(o) == -1);
	}
	inline bool operator==(const Object& o) const {
		return (compare(o) == 0);
	}
	inline bool operator!=(const Object& o) const {
		return (compare(o) == 1);
	}

public:

	virtual const Type& getType() const;
	const AttributeValue& getAttributeValue(const std::string& name) const;
	inline const AttributeValuesMap& getAttributeValues() const { return attributes; }

	inline virtual bool isDatabaseObject() {
		return false;
	};
	/*inline void setTransient() {
		IOPC_DEBUGLOG("Object::setTransient");
	};

	inline void freeAll() throw () {
		IOPC_DEBUGLOG("Object::freeAll");
	};

	inline void initAll() throw () {
		IOPC_DEBUGLOG("Object::initAll");
	};*/

	virtual Object* clone() const;


	virtual ~Object() {};

protected:
	virtual const Type& getTypeInternal() const;
};

}

#endif /*OBJECT_H_*/
