#include <iostream>
#include "testcase.h"
#include "iopccommon/trace.h"
#include <sstream>


TestCase::TestCase(const std::string& testName, bool initTest, bool tearDownTest)
{
	this->name = testName;
	if (initTest) {
		TestCase::initTest = this;
		return;
	}
	if (tearDownTest) {
		TestCase::tearDownTest = this;
		return;
	}
	
	TestCase::addTest(this);
}


TestCase::~TestCase()
{
	if (this == initTest) {
		TestCase::initTest = NULL;
		return;
	}
	if (this == tearDownTest) {
		TestCase::tearDownTest = NULL;
		return;
	}
	
	TestCase::removeTest(this);
}


// static methods
/**
 * Workaround for static initialization dependency problem
 */
std::set<TestCase*>& TestCase::getTestCases()
{
	static std::set<TestCase*> testCases;
	return testCases;
}

void TestCase::runTests()
{
	if (initTest) {
		std::cout << "------------------------------" << std::endl;
		std::cout << "Running test INIT... " << std::endl;
		iopc::LogWriter::logInfo("------ TEST INIT ------");
		initTest->doTest();
	}
	
	for (std::set<TestCase*>::iterator i = getTestCases().begin(); i != getTestCases().end(); i++) {
		std::cout << "------------------------------" << std::endl;
		std::cout << "Running test: " << (*i)->name << "... " << std::endl;
		iopc::LogWriter::logInfo("------ TEST: " + (*i)->name + " ------");
		
		try {
			if ((*i)->doTest()) {
				std::cout << "SUCCESS" << std::endl;
			} else {
				std::cout << "FAILED" << std::endl;
			}
		} catch (std::exception& e) {
			std::cout << e.what() << std::endl;
			std::cout << "FAILED" << std::endl;
		}
		
		std::cerr.flush();
		std::cout.flush();
	}
	
	if (tearDownTest) {
		tearDownTest->doTest();
	}
}

void TestCase::logMsg(const std::string& msg)
{
	//std::cout << msg << std::endl;
	iopc::LogWriter::logInfo("TEST: " + msg);
}


void TestCase::logMsg(const std::string& msg, const int val)
{
	std::ostringstream sstr;
	sstr << msg << " " << val;
	//std::cout << sstr.str() << std::endl;
	iopc::LogWriter::logInfo("TEST: " + sstr.str());
}

void TestCase::addTest(TestCase* tc)
{
	getTestCases().insert(tc);
}

void TestCase::removeTest(TestCase* tc)
{
	getTestCases().erase(tc);
}

TestCase* TestCase::initTest = NULL;
TestCase* TestCase::tearDownTest = NULL;
