#include "iopccommon/trace.h"
#include "tests/associationTest.h"
#include "iopclib/dbPtr.h"
#include "iopcdb/driverManager.h"
#include "iopcdb/cursor.h"
#include "iopclib/types/dbTypeManager.h"
#include "iopclib/db/cachedDatabase.h"
#include "iopclib/db/cachedConnection.h"
#include "iopclib/cache/voidCache.h"
#include "iopclib/cache/lruCacheST.h"
#include "iopclib/cache/arcCacheST.h"

#include "testSchema/testObject.h"



using namespace iopc;
using namespace std;

AssociationTest::AssociationTest(const std::string& testName)
 : TestCase(testName)
{

}

AssociationTest::~AssociationTest()
{
}


bool AssociationTest::doTest()
{
	Database* db = NULL;
	CachedConnection* conn = NULL;
	bool ret = true;
	try {
		db = new CachedDatabase(DriverManager::getDriver("IopcOracle10g").getDatabase(DBNAME), new HashArcCacheST());
		conn = (CachedConnection*)db->getConnection(CONNSTR);
		conn->open();
		conn->setAutoCommit(false);

		DbPtr<Person> person;
		person->surname = "Karel";
		person.bePersistent(conn);
		DbPtr<Person> person2;
		person2->surname = "Petr";
		person2.bePersistent(conn);
		OidType oid = person2.getPersId().getOid();
		logMsg("Person->oid ", oid);

		DbPtr<Company> company;
		company->name = "IBM";
		company->owner = person2;

		OidType oid2 = ((DbPtrBase*)company->getAttributeValue("owner").getValue())->getPersId().getOid();
		logMsg("Company->owner->oid", oid);
		if (oid != oid2) {
			ret=false;
		}
		EString surname1 = company->owner->surname;
		logMsg(surname1);
		if (surname1 != "Petr") {
			ret=false;
		}

		company.bePersistent(conn);

		EString surname2 = company->owner->surname;
		logMsg(surname2);
		if (surname2 != "Petr") {
			ret=false;
		}
		conn->updateDirty();
		conn->removeAll();

		EString surname3 = company->owner->surname;
		logMsg(surname3);
		if (surname3 != "Petr") {
			ret=false;
		}
		DbPtr<Person> person3(conn, oid);
		logMsg(person3->surname);

		// update
		company->owner = person; // "Karel"
		conn->updateDirty();
		conn->removeAll();
		EString surname4 = company->owner->surname;
		logMsg(surname4);
		if (surname4 != "Karel") {
			ret=false;
		}


		//OidType oid3 = company->owner.getPersId().getOid();
		//logMsg("oid: ", oid3);

		/*OidType oid2 = ((DbPtrBase*)attrVal.getValue())->getPersId().getOid();
		logMsg("Company->owner->oid", oid);
		logMsg(company->owner->surname);
		if (oid != oid2) {
			ret=false;
		}*/


		conn->commit();

	} catch (IopcException& e) {
		LogWriter::logError(e.what());
		ret = false;
	}

	if (db && conn) db->returnConnection(conn);
	if (db) DriverManager::getDriver("IopcOracle10g").returnDatabase(db);

	return ret;
}

static AssociationTest AssociationTestInstance("Associations");
