#include "tests/queryTest.h"
#include "iopccommon/trace.h"
#include "iopcdb/driverManager.h"
#include "iopclib/db/cachedDatabase.h"
#include "iopclib/db/cachedConnection.h"
#include "testSchema/diamond.h"
#include "iopclib/cache/composedCache.h"
#include "iopclib/cache/cacheKeeper.h"
#include "iopclib/cache/voidCache.h"
#include "iopclib/cache/lruCache.h"
#include "iopclib/cache/arcCache.h"
#include "iopclib/dbPtr.h"
#include "iopclib/result.h"
#include "iopclib/simpleQuery.h"
#include "iopclib/freeQuery.h"
#include "testSchema/fakeType.h"
#include "testSchema/fakeObject.h"
#include "testSchema/testObject.h"
#include "testSchema/diamond.h"
#include <vector>

//#include "testSchema/fakeObject.h"
//#include "testSchema/fakeType.h"

using namespace iopc;
using namespace std;

QueryTest::QueryTest(const std::string& testName)
 : TestCase(testName)
{

}

QueryTest::~QueryTest()
{
}


bool QueryTest::doTest()
{
	Database* db = NULL;
	CachedConnection* conn = NULL;
	bool ret = true;
	try {
		db = new CachedDatabase(DriverManager::getDriver("IopcOracle10g").getDatabase(DBNAME), new ComposedCache());
		conn = (CachedConnection*)db->getConnection(CONNSTR);
		conn->open();
		conn->setAutoCommit(false);

		CacheKeeper cacheKeeper(-15000, 20, 40);
		cacheKeeper.setCache(conn, new HashArcCache());
		//cacheKeeper.setStrategy(*conn, Strategy::);
		cacheKeeper.runMaintainer();

		DbPtr<TestObject> testObject;
		testObject->attrInt = 1;
		testObject->attrDouble = 1.3;
		testObject->attrString = "QueryTest1";
		//logMsg("MemCost :", (int) testObject->getMemCost());
		testObject.bePersistent(conn);

		DbPtr<DiamondClass9> testObj2;
		testObj2->test9 = "QueryTest2";
		testObj2.bePersistent(conn);

		DbPtr<DiamondClass9> testObj3;
		testObj3->test9 = "QueryTest3";
		testObj3.bePersistent(conn);

		logMsg("Simple query...");

		SimpleQuery qTest("$::DiamondClass9[db.view.pv]::test9$ = 'QueryTest2' AND $$$brmn$");
		string qSql = qTest.getSql(conn->getDriver(), TypeDesc<DiamondClass9>::getType());
		if (qSql != "SELECT OID FROM diamond9_PV WHERE diamond9_PV.diamond9_test9 = 'QueryTest2' AND $brmn") {
			logMsg("qTest.getSql failed");
			ret = false;
		}
		logMsg(qSql);

		SimpleQuery q;
		Result<DiamondClass9> r(conn, q, false);
		//Result<TestObject> r2(conn, q);
		r.open();
		//r2.open();
		for (Result<DiamondClass9>::iterator it = r.begin(); it != r.end(); it++) {
			logMsg("result->test9: " + it->test9);
		}

		r.close();


		logMsg("Free query...");
		FreeQuery qFree1("$::DiamondClass9[db.view.pv]$ x WHERE x$::DiamondClass9[db.query.skip]::test6[db.viewColumn]$ = 'value 6'", "x");
		qSql = qFree1.getSql(conn->getDriver(), TypeDesc<DiamondClass9>::getType());
		if (qSql != "SELECT x.OID FROM diamond9_PV x WHERE x.diamond6_test6 = 'value 6'") {
			logMsg("qFree1.getSql failed");
			ret = false;
		}
		Result<DiamondClass9> rFree1(conn, qFree1);
		//Result<TestObject> r2(conn, q);
		rFree1.open();
		//r2.open();
		for (Result<DiamondClass9>::iterator it = rFree1.begin(); it != rFree1.end(); it++) {
			logMsg("result->test9: " + it->test9);
		}
		rFree1.close();


		logMsg("Free query, transient objects");
		Result< ::UserTable> rUserTable(conn, FreeQuery("user_tables where table_name like 'DIA%'"));
		rUserTable.open();
		for (Result< ::UserTable>::iterator it = rUserTable.begin(); it != rUserTable.end(); it++) {
			logMsg("tableName: " + it->tableName);
			if (!it->blocks.isNull()) {
				logMsg("blocks: ", it->blocks);
			}
		}
		rUserTable.close();

		conn->commit();

		cacheKeeper.stopMaintainer();
	} catch (IopcException& e) {
		LogWriter::logError(e.what());
		ret = false;
	}

	if (db && conn) db->returnConnection(conn);
	if (db) DriverManager::getDriver("IopcOracle10g").returnDatabase(db);

	return ret;
}

static QueryTest dbPtrTestInstance("Query Test");
